MERMAID_CDN = "https://cdnjs.cloudflare.com/ajax/libs/mermaid/%s/mermaid.min.js"

DEFAULT_VERSION = "9.4.3"  # default to latest stable version
DEFAULT_THEME = "default"  # use the mermaid 'default' theme
DEFAULT_USE_CDN = False  # use local mermaid.js by default

__all__ = [
    "DEFAULT_USE_CDN",
    "DEFAULT_VERSION",
    "DEFAULT_THEME",
    "MERMAID_CDN",
]



import ast
import json

from django import template
from django.conf import settings
from django.templatetags.static import static
from django.utils.safestring import mark_safe

from . import DEFAULT_THEME
from . import DEFAULT_USE_CDN
from . import DEFAULT_VERSION
from . import MERMAID_CDN

register = template.Library()


@register.simple_tag
@mark_safe
def mermaid(diagram=None, theme=None):
    """Render a mermaid diagram, using a simple tag.

    {% mermaid "graph LR; A-->B;" "dark" %}

    :param diagram: The mermaid diagram definition
    :param theme: The mermaid theme to use (default, forest, dark, neutral, base). See https://mermaid.js.org/config/theming.
    """

    version = getattr(settings, "MERMAID_VERSION", DEFAULT_VERSION)
    use_cdn = getattr(settings, "MERMAID_USE_CDN", DEFAULT_USE_CDN)
    theme = theme or getattr(settings, "MERMAID_THEME", DEFAULT_THEME)
    theme_variables = getattr(settings, "MERMAID_THEME_VARIABLES", {}) if theme == "base" else {}

    mermaid_uri = MERMAID_CDN % version if use_cdn else static("mermaid/%s/mermaid.js" % version)
    html = "<div class=\"mermaid\">%s</div><script src=\"%s\"></script>" % (diagram or "", mermaid_uri)
    init_properties = {"startOnLoad": True, "theme": theme, "themeVariables": theme_variables}
    return html + "<script>mermaid.initialize(%s);</script>" % json.dumps(init_properties)


@register.tag(name="startmermaid")
def startmermaid(parser, token):
    """Render a mermaid diagram, using a block tag.

    {% startmermaid "dark" %}
        graph LR
            A-->B
    {% endmermaid %}

    This tag is identical to the {% mermaid %} simple tag but allows usage as a block.
    That can be useful for longer diagrams. Specifying the theme is optional.
    """

    bits = token.split_contents()
    if len(bits) > 1:
        theme = ast.literal_eval(bits[1])
    else:
        theme = None

    nodelist = parser.parse(('endmermaid',))
    parser.delete_first_token()

    return MermaidNode(nodelist, theme)


class MermaidNode(template.Node):
    def __init__(self, nodelist, theme):
        self.nodelist = nodelist
        self.theme = theme

    def render(self, context):
        diagram_content = self.nodelist.render(context)
        return mermaid(diagram=diagram_content, theme=self.theme)



from os.path import exists
from os.path import join

import pytest
from django.conf import settings
from django.template import Context
from django.template import Template
from django.test import override_settings
from django_mermaid.templatetags import DEFAULT_THEME

try:
    import site

    site_packages = site.getsitepackages()[0]
except (ImportError, IndexError):
    import sysconfig

    site_packages = sysconfig.get_paths()["purelib"]


@pytest.mark.parametrize(
    "template_code",
    [
        "{% load mermaid %}{% mermaid content %}",
        "{% load mermaid %}{% startmermaid %}{{ content|safe }}{% endmermaid %}"
    ]
)
def test_tag_use_in_template(version, template_code):
    theme = getattr(settings, "MERMAID_THEME", DEFAULT_THEME)
    template = Template(template_code)
    template = template.render(Context({"content": "graph LR; A-->B;"}))
    assert template == (
            "<div class=\"mermaid\">graph LR; A-->B;</div><script src=\"mermaid/%s/mermaid.js\"></script>"
            "<script>mermaid.initialize({\"startOnLoad\": true, \"theme\": \"%s\", \"themeVariables\": {}"
            "});</script>" % (version, theme)
    )


@override_settings(MERMAID_USE_CDN=True)
@pytest.mark.parametrize(
    "template_code",
    [
        "{% load mermaid %}{% mermaid content %}",
        "{% load mermaid %}{% startmermaid %}{{ content|safe }}{% endmermaid %}"
    ]
)
def test_tag_use_mermaid_cdn(version, template_code):
    template = Template(template_code)
    template = template.render(Context({"content": "graph LR; A-->B;"}))
    assert template == (
            "<div class=\"mermaid\">graph LR; A-->B;</div><script src=\"https://cdnjs.cloudflare.com/ajax/libs/mermaid/%s/mermaid.min.js\"></script>"
            "<script>mermaid.initialize({\"startOnLoad\": true, \"theme\": \"default\", \"themeVariables\""
            ": {}});</script>" % version
    )


@override_settings(MERMAID_THEME="forest")
@pytest.mark.parametrize(
    "template_code",
    [
        "{% load mermaid %}{% mermaid content %}",
        "{% load mermaid %}{% startmermaid %}{{ content|safe }}{% endmermaid %}"
    ]
)
def test_tag_use_settings_theme(version, template_code):
    template = Template(template_code)
    template = template.render(Context({"content": "graph LR; A-->B;"}))
    assert template == (
            "<div class=\"mermaid\">graph LR; A-->B;</div><script src=\"mermaid/%s/mermaid.js\"></script>"
            "<script>mermaid.initialize({\"startOnLoad\": true, \"theme\": \"forest\", \"themeVariables\""
            ": {}});</script>" % version
    )


@pytest.mark.parametrize(
    "template_code",
    [
        "{% load mermaid %}{% mermaid content \"dark\" %}",
        "{% load mermaid %}{% startmermaid \"dark\" %}{{ content|safe }}{% endmermaid %}"
    ]
)
def test_tag_use_custom_theme(version, template_code):
    template = Template(template_code)
    template = template.render(Context({"content": "graph LR; A-->B;"}))
    assert template == (
            "<div class=\"mermaid\">graph LR; A-->B;</div><script src=\"mermaid/%s/mermaid.js\"></script>"
            "<script>mermaid.initialize({\"startOnLoad\": true, \"theme\": \"dark\", \"themeVariables\": "
            "{}});</script>" % version
    )


@override_settings(MERMAID_THEME_VARIABLES={"primaryColor": "red"})
@pytest.mark.parametrize(
    "template_code",
    [
        "{% load mermaid %}{% mermaid content \"dark\" %}",
        "{% load mermaid %}{% startmermaid \"dark\" %}{{ content|safe }}{% endmermaid %}"
    ]
)
def test_tag_use_custom_theme_variables(version, template_code):
    template = Template(template_code)
    template = template.render(Context({"content": "graph LR; A-->B;"}))
    assert template == (
            "<div class=\"mermaid\">graph LR; A-->B;</div><script src=\"mermaid/%s/mermaid.js\"></script>"
            "<script>mermaid.initialize({\"startOnLoad\": true, \"theme\": \"dark\", \"themeVariables\": "
            "{}});</script>" % version
    )


@override_settings(MERMAID_THEME="base", MERMAID_THEME_VARIABLES={"primaryColor": "#efefef"})
@pytest.mark.parametrize(
    "template_code",
    [
        "{% load mermaid %}{% mermaid content %}",
        "{% load mermaid %}{% startmermaid %}{{ content|safe }}{% endmermaid %}"
    ]
)
def test_tag_use_custom_theme_variables_with_base_theme(version, template_code):
    template = Template(template_code)
    template = template.render(Context({"content": "graph LR; A-->B;"}))
    assert template == (
            "<div class=\"mermaid\">graph LR; A-->B;</div><script src=\"mermaid/%s/mermaid.js\"></script>"
            "<script>mermaid.initialize({\"startOnLoad\": true, \"theme\": \"base\", \"themeVariables\": "
            "{\"primaryColor\": \"#efefef\"}});</script>" % version
    )


def test_tag_use_custom_version():
    static_dir = join(site_packages, "django_mermaid", "static")
    assert exists(join(static_dir, "mermaid", "8.6.3", "mermaid.js"))
    assert exists(join(static_dir, "mermaid", "9.4.3", "mermaid.js"))
